'****************************************************************
'*  Name    : CAN_INTERRUPT_INC.INC                             *
'*  Author  : Bobby Garrett                                     *
'*  Notice  : Copyright (c) 2012 Bobby Garrett                  *
'*          : All Rights Reserved                               *
'*  Date    : 21/08/2012                                        *
'*  Version : 1.0                                               *
'*  Notes   :                                                   *
'*          :                                                   *
'****************************************************************

'****************************************************************
'* Interrupt service vector initialization                      *
'****************************************************************

        On_Hardware_Interrupt GoTo HIGH_INTERRUPT_ROUTINE    
        On_Low_Interrupt GoTo LOW_INTERRUPT_ROUTINE 
                                                                  
        GoTo OVER_INT  

'****************************************************************       
'* Interrupt Service Routine                                    *
'****************************************************************
'* Determine if it is a timed interrupt (LOW priority)          *
'* Or CAN module Interrupt (HIGH priority) has occured.         *
'* The ISR determines whether it is a Valid message or          *
'* CAN error interrupt and then proceeds accordingly.           *
'****************************************************************

HIGH_INTERRUPT_ROUTINE:
        Context Save            
        
        If PIR3.0 = 1 Then
        GoSub CAN_RECEIVE
        PIR3 = 0               'Clear CAN bus error interrupt bits ? IS THIS CORRECT ? in RX routine
        EndIf
       
        If PIR3.1 = 1 Then
        GoSub CAN_RECEIVE
        PIR3 = 0               'Clear CAN bus error interrupt bits ? IS THIS CORRECT ? in RX routine
        EndIf        
                
      ''  If RXB0CON.7 = 1 Then GoTo CAN_RECEIVE 
      ''  If RXB1CON.7 = 1 Then GoTo CAN_RECEIVE 
         
        Include "C:\JB_Test\CAN_RX_INC.INC"
       
'****************************************************************
'* CAN Error routine   !!!!! to complete                        *
'* If the CAN module detects a CAN error or RX buffer 1 has     *
'* overflowed then process condition and exit.                  *
'* The If/Then statements are in order of priority in case      *
'* there are more than one. The worst case is first priority.   *
'*                                                              *
'* Counters are incremented for diagnosis and are sent out      *
'* later on in the program.                                     *
'****************************************************************

        If PIR3.5 = 0 Then GoTo No_CAN_Errors

'TX_IS_BUS_OFF
        If COMSTAT.5 = 1 Then
        COMSTAT.5 = 0
        'Note to me !! reset bits and clear anything else needed
        EndIf

'TX_IS_BUS_PASSIVE        
        If COMSTAT.4 = 1 Then
        COMSTAT.4 = 0
        'Note to me !! reset bits and clear anything else needed
        EndIf
        
'RX_IS_BUS_PASSIVE         
        If COMSTAT.3 = 1 Then
        COMSTAT.3 = 0
        'Note to me !! reset bits and clear anything else needed
        EndIf 
        
'CAN_RX_OVERFLOW_ERROR               
        If COMSTAT.6 = 1 Then
        COMSTAT.6 = 0
        'Note to me !! reset bits and clear anything else needed
        EndIf

'TX_WARNING        
        If COMSTAT.2 = 1 Then
        COMSTAT.2 = 0   'Probably leave as it should correct itself ??
        'Note to me !! reset bits and clear anything else needed
        EndIf
        
'RX_WARNING        
        If COMSTAT.1 = 1 Then
        COMSTAT.1 = 0   'Probably leave as it should correct itself ??
        'Note to me !! reset bits and clear anything else needed
        EndIf

EXIT_CAN_ERRORS:        
        COMSTAT = 0
        PIR3.5 = 0             'Clear CAN error Interrupt bit
        'Note to me !! reset bits and clear anything else needed        
        
No_CAN_Errors:
        
'End of CAN errors Interrupt       
        
        Context Restore          'Exit ISR and re-enable interrupts  

LOW_INTERRUPT_ROUTINE:
        Context Save            
'Timer interrupt, set a flag so various counters get incremented outside the interrupt.
        If PIR1.0 = 1 Then 
        Clear T1CON.0            'Stop timer
        TIMER_COUNT_REQ = 1        
        
        'Set / clear relevant bits before returning from interupt
        PIR1.0 = 0               'Clear Timer 1 overflow interrupt bit
        TIMER1 = TIMER_1_OFFSET  'Re-load timer
        Set T1CON.0              'Re-start timer
        EndIf       

        Context Restore          'Exit ISR and re-enable interrupts  

OVER_INT:         
        
'****************************************************************
'* End of Interrupt routines                                    *
'****************************************************************        
        
