'****************************************************************
'*  Name    : CAN_RX_INC.INC                                    *
'*  Author  : Bobby Garrett                                     *
'*  Notice  : Copyright (c) 2012 Bobby Garrett                  *
'*          : All Rights Reserved                               *
'*  Date    : 21/08/2012                                        *
'*  Version : 1.0                                               *
'*  Notes   :                                                   *
'*          :                                                   *
'****************************************************************


'****************************************************************            
'* Variables for CAN RX                                         *
'**************************************************************** 
    
        Dim RX_NODE_NR As Word        
        Dim RX_NODE_ID[4] As Byte
        Dim RX_MSG_ID As Byte
        Dim RX_DATA[8] As Byte
        Dim RX_MSG_LEN[1] As Byte   'Created array of 1 byte so I can pass 
                                    'data back out of CANReceiveMessage routine 
        Dim RX_FLAGS As Byte

        Dim RX_LOOP_COUNTER As Byte     'Loop counter for storing RX Data

'****************************************************************            
'* Variables for RX data storage                                *
'* Name arrays as suitable for project and comment out or       *
'* delete what are not required.                                *
'* Change name used for storage later in program.               *
'* Example - change Dim MESSAGE_0_DATA#8 As Byte '000000 00001  *
'*               to Dim SPEED#8 As Byte  '000000 00001          *
'* and change  VarPtr(MESSAGE_0_DATA) to VarPtr(SPEED)          *
'* in store routine.                                            *
'****************************************************************           
'* You must have an array for each possible message received    *
'* you do not need an array that corresponds to this node       *
'* IE if this is node 2 then it will not receive messages from  *
'* node 2.                                                      *
'* This code is generic, rename and remove what is not required *
'****************************************************************
'* Also see the CAN_RECEIVE subroutine, you need the following  *
'* If RX_NODE_NR = 1 Then GoTo MESSAGE_FROM_NODE_1              *
'* and                                                          *
'  MESSAGE_FROM_NODE_1:                                         *
'          FSReg1 = 0                                           *
'          If RX_MSG_ID = 0 Then FSReg1 = VarPtr(MESSAGE_0_DATA) : GoTo STORE_RX_MESSAGES       
'          If RX_MSG_ID = 1 Then FSReg1 = VarPtr(MESSAGE_1_DATA) : GoTo STORE_RX_MESSAGES  
'****************************************************************

        Dim MAX_DATA As Byte           'No of data bytes in RX message 

'The message and node bits are actually determined by the store
'routine so are only valid for this example               
'Messages from Node 1                    Msg   Node 
        Dim MESSAGE_0_DATA[8] As Byte  '000000 00001             
        Dim MESSAGE_1_DATA[8] As Byte  '000001 00001
        Dim MESSAGE_2_DATA[8] As Byte  '000010 00001
        Dim MESSAGE_3_DATA[8] As Byte  '000011 00001
        Dim MESSAGE_4_DATA[8] As Byte  '000100 00001
        
'Messages from Node 2                    Msg   Node
        Dim MESSAGE_5_DATA[8] As Byte  '000000 00010
        Dim MESSAGE_6_DATA[8] As Byte  '000001 00010
        Dim MESSAGE_7_DATA[8] As Byte  '000010 00010
        Dim MESSAGE_8_DATA[8] As Byte  '000011 00010
        Dim MESSAGE_9_DATA[8] As Byte  '000100 00010
                
'Messages from Node 3                    Msg   Node
        Dim MESSAGE_10_DATA[8] As Byte '000000 00011
        Dim MESSAGE_11_DATA[8] As Byte '000001 00011
        Dim MESSAGE_12_DATA[8] As Byte '000010 00011
        Dim MESSAGE_13_DATA[8] As Byte '000011 00011
        Dim MESSAGE_14_DATA[8] As Byte '000100 00011       

'Messages from Node 4                    Msg   Node
        Dim MESSAGE_15_DATA[8] As Byte '000000 00100
        Dim MESSAGE_16_DATA[8] As Byte '000001 00100
        Dim MESSAGE_17_DATA[8] As Byte '000010 00100
        Dim MESSAGE_18_DATA[8] As Byte '000011 00100
        Dim MESSAGE_19_DATA[8] As Byte '000100 00100       

'Messages from Node 5                    Msg   Node
        Dim MESSAGE_20_DATA[8] As Byte '000000 00101
        Dim MESSAGE_21_DATA[8] As Byte '000001 00101
        Dim MESSAGE_22_DATA[8] As Byte '000010 00101
        Dim MESSAGE_23_DATA[8] As Byte '000011 00101
        Dim MESSAGE_24_DATA[8] As Byte '000100 00101     
        
'****************************************************************        
'* CAN_RECEIVE                                                  *
'* After the CANReceiveMessage routine returns the message the  *
'* sending Node ID and message ID is determined. DATA is then   *
'* stored in the relevant array.                                *
'* !!!! Only standard identifiers used - 5 bits for Node ID and *
'* 6 bits for the Message ID (32 Node ID's and 64 Messages).    *
'****************************************************************
CAN_RECEIVE:
        Toggle RX_LED      'Flash LED - let me know it is receiving
        HRSOut "Node 1 RX Routine",10,10, 13
        CANReceiveMessage [RX_NODE_ID,RX_DATA,RX_MSG_LEN,RX_FLAGS]

'Determine Node ID
'Only STD message ID done in this example
'RX_NODE_ID#0/#1 = RXBxSIDH/L
        RX_NODE_NR.HighByte = RX_NODE_ID#0
        RX_NODE_NR.LowByte = RX_NODE_ID#1
        RX_NODE_NR = RX_NODE_NR >> 5
        RX_NODE_NR = RX_NODE_NR & %0000000000011111 'Mask higher 11 bits
'Test for duplicate Node ID on the BUS
        If RX_NODE_NR = TX_NODE_NR Then GoTo CAN_DUPLICATE_ERROR         

'Determine Message ID 
'Only STD message ID done in this example
'RX_NODE_ID#0 = RXBxSIDH       
        RX_MSG_ID = RX_NODE_ID#0
        RX_MSG_ID = RX_MSG_ID >> 2
       
'Store data in relevant array               
        MAX_DATA = RX_MSG_LEN#0 & %00001111   'strip RTR bit 
        
        FSReg0 = VarPtr(RX_DATA) 

'ADD/DELETE AS NECESSARY        
' Determine which was the transmitting node and store data accordingly
' See comments in "Variables For Data storage" regarding 
' Array names IE "MESSAGE_0_DATA" could be "SPEED" etc.
' Message ID's do not have to be sequential

        If RX_NODE_NR = 1 Then GoTo MESSAGE_FROM_NODE_1  
        If RX_NODE_NR = 2 Then GoTo MESSAGE_FROM_NODE_2       
        If RX_NODE_NR = 3 Then GoTo MESSAGE_FROM_NODE_3                   
        If RX_NODE_NR = 4 Then GoTo MESSAGE_FROM_NODE_4
        If RX_NODE_NR = 5 Then GoTo MESSAGE_FROM_NODE_5
'If program gets to here there is an issue so goto RX_MESSAGE_ERROR and log it  
        GoTo RX_MESSAGE_ERROR
      
MESSAGE_FROM_NODE_1:                
        FSReg1 = 0 
        If RX_MSG_ID = 0 Then FSReg1 = VarPtr(MESSAGE_0_DATA) : GoTo STORE_RX_MESSAGES       
        If RX_MSG_ID = 1 Then FSReg1 = VarPtr(MESSAGE_1_DATA) : GoTo STORE_RX_MESSAGES      
        If RX_MSG_ID = 2 Then FSReg1 = VarPtr(MESSAGE_2_DATA) : GoTo STORE_RX_MESSAGES                  
        If RX_MSG_ID = 3 Then FSReg1 = VarPtr(MESSAGE_3_DATA) : GoTo STORE_RX_MESSAGES
        If RX_MSG_ID = 4 Then FSReg1 = VarPtr(MESSAGE_4_DATA) : GoTo STORE_RX_MESSAGES
'If program gets to here there is an issue so goto RX_MESSAGE_ERROR and log it  
        GoTo RX_MESSAGE_ERROR
        
MESSAGE_FROM_NODE_2:                
        FSReg1 = 0
        If RX_MSG_ID = 0 Then FSReg1 = VarPtr(MESSAGE_5_DATA) : GoTo STORE_RX_MESSAGES       
        If RX_MSG_ID = 1 Then FSReg1 = VarPtr(MESSAGE_6_DATA) : GoTo STORE_RX_MESSAGES       
        If RX_MSG_ID = 2 Then FSReg1 = VarPtr(MESSAGE_7_DATA) : GoTo STORE_RX_MESSAGES                  
        If RX_MSG_ID = 3 Then FSReg1 = VarPtr(MESSAGE_8_DATA) : GoTo STORE_RX_MESSAGES
        If RX_MSG_ID = 4 Then FSReg1 = VarPtr(MESSAGE_9_DATA) : GoTo STORE_RX_MESSAGES
'If program gets to here there is an issue so goto RX_MESSAGE_ERROR and log it  
        GoTo RX_MESSAGE_ERROR
        
MESSAGE_FROM_NODE_3:                
        FSReg1 = 0 
        If RX_MSG_ID = 0 Then FSReg1 = VarPtr(MESSAGE_10_DATA) : GoTo STORE_RX_MESSAGES       
        If RX_MSG_ID = 1 Then FSReg1 = VarPtr(MESSAGE_11_DATA) : GoTo STORE_RX_MESSAGES     
        If RX_MSG_ID = 2 Then FSReg1 = VarPtr(MESSAGE_12_DATA) : GoTo STORE_RX_MESSAGES                  
        If RX_MSG_ID = 3 Then FSReg1 = VarPtr(MESSAGE_13_DATA) : GoTo STORE_RX_MESSAGES
        If RX_MSG_ID = 4 Then FSReg1 = VarPtr(MESSAGE_14_DATA) : GoTo STORE_RX_MESSAGES
'If program gets to here there is an issue so goto RX_MESSAGE_ERROR and log it    
        GoTo RX_MESSAGE_ERROR
        
MESSAGE_FROM_NODE_4:                
        FSReg1 = 0 
        If RX_MSG_ID = 0 Then FSReg1 = VarPtr(MESSAGE_15_DATA) : GoTo STORE_RX_MESSAGES       
        If RX_MSG_ID = 1 Then FSReg1 = VarPtr(MESSAGE_16_DATA) : GoTo STORE_RX_MESSAGES     
        If RX_MSG_ID = 2 Then FSReg1 = VarPtr(MESSAGE_17_DATA) : GoTo STORE_RX_MESSAGES                  
        If RX_MSG_ID = 3 Then FSReg1 = VarPtr(MESSAGE_18_DATA) : GoTo STORE_RX_MESSAGES
        If RX_MSG_ID = 4 Then FSReg1 = VarPtr(MESSAGE_19_DATA) : GoTo STORE_RX_MESSAGES
'If program gets to here there is an issue so goto RX_MESSAGE_ERROR and log it   
        GoTo RX_MESSAGE_ERROR

MESSAGE_FROM_NODE_5:                
        FSReg1 = 0 
        If RX_MSG_ID = 0 Then FSReg1 = VarPtr(MESSAGE_20_DATA) : GoTo STORE_RX_MESSAGES       
        If RX_MSG_ID = 1 Then FSReg1 = VarPtr(MESSAGE_21_DATA) : GoTo STORE_RX_MESSAGES     
        If RX_MSG_ID = 2 Then FSReg1 = VarPtr(MESSAGE_22_DATA) : GoTo STORE_RX_MESSAGES                  
        If RX_MSG_ID = 3 Then FSReg1 = VarPtr(MESSAGE_23_DATA) : GoTo STORE_RX_MESSAGES
        If RX_MSG_ID = 4 Then FSReg1 = VarPtr(MESSAGE_24_DATA) : GoTo STORE_RX_MESSAGES
'If program gets to here there is an issue so goto RX_MESSAGE_ERROR and log it   
        GoTo RX_MESSAGE_ERROR

STORE_RX_MESSAGES:                   
        For RX_LOOP_COUNTER = 0 To MAX_DATA -1               
        POSTINC1 = POSTINC0     'Copy data from Receive buffer 
        Next RX_LOOP_COUNTER                       
        Return        
                
'****************************************************************
'* CAN RX Error routine - Invalid message detected              *
'* If the program get to here it is because either the received *
'* node/message has not been decoded correctly or more likely   *
'* that there is no pointer to an array for it to go to.        *
'* I have not decided how to handle this condition yet, this is *
'* just to trap the error so I know it has occured.             *
'* In practice, this could happen as the receiving node may not *
'* be interested in the transmitted data so no pointer for it.  *
'****************************************************************

RX_MESSAGE_ERROR:     
        
'Inc count and save in EEPROM.        
        
'End of CAN RX Interrupt            
