'****************************************************************
'*  Name    : RGV CAN BUS test                                  *
'*  Author  : Bobby Garrett                                     *
'*  Notice  : Copyright (c) 2012 Bobby Garrett                  *
'*          : All Rights Reserved                               *
'*  Date    : 21/08/2012                                        *
'*  Version : 1                                                 *
'*  Library : CAN_MACROS_1_4                                    *
'*  Node    : 1                                                 *
'*  Notes   : CAN BUS Set For 125 kb/s                          *
'*          :                                                   *
'*          : Only STD ID's are used !!                         *
'**************************************************************** 
'****************************************************************          
'* Select the PIC configuration, only Select one Device         *
'**************************************************************** 

        Include "C:\JB_Test\18F258_CONFIG.INC"

        Xtal = 20                   'Define clock
        All_Digital = True          'Set all I/O to digital
        Stack_Size = 20             'Stack for TX queue       
        
        Clear                       'Clear all RAM
        DelayMS 100                 'Let PIC start up  

        Dim FSReg0 As FSR0L.Word
        Dim FSReg1 As FSR1L.Word

'****************************************************************            
'* Variables for CAN TX                                         *
'**************************************************************** 
        Dim TX_NODE_NR As Word      'Used for - Start up delay
                                    '         - Start up message
                                    '         - Duplicate node ID check

'****************************************************************            
'* Variables for CAN TX                                         *
'****************************************************************    
        Dim MSGID_DATA As Dword
        Dim MSGID_TYPE As Byte  

'****************************************************************            
'* Variables                                                    *
'****************************************************************        
        Dim H_BEAT As Byte      'Heartbeat counter to check comms OK

'****************************************************************            
'* Diagnostic LED's                                             *
'****************************************************************           
        Symbol LOOP_LED = PORTA.0       'Alias the GREEN LED (LOOP)       
        Symbol TX_LED = PORTA.1         'Alias the AMBER LED (TX) 
        Symbol CAN_ERROR_LED = PORTA.2  'Alias the RED LED (CAN error) 
        Symbol RX_LED = PORTA.3         'Alias the AMBER LED (RX)  
        
'Set port direction On power up Or after reset        
        TRISA.0 = 0             'Set port direction - LED for diagnostics
        TRISA.1 = 0             'Set port direction - LED for diagnostics
        TRISA.2 = 0             'Set port direction - LED for diagnostics
        TRISA.3 = 0             'Set port direction - LED for diagnostics        
'Set state of port on power up or after reset
        PORTA.0 = 0             'Initialise output as off
        PORTA.1 = 0             'Initialise output as off
        PORTA.2 = 0             'Initialise output as off
        PORTA.3 = 0             'Initialise output as off                  

'CCP Setup
'       TRISC.2 = 0 'CCP not used so port can be used normally if required
' Disable CCP 
        CCP1CON = %00000000               
                                  
'****************************************************************            
'* Variables for Timer1                                         *
'****************************************************************                                       
        Dim TIMER1 As TMR1L.Word
        Dim TIMER_1_BITS As Byte
        Dim TIMER_1_OFFSET As Word

'****************************************************************            
'* Symbols for Interrupts and LED's etc                         *
'****************************************************************  
        Symbol IPEN = RCON.7
        Symbol TMR1IP = IPR1.0   
        Symbol TMR1IF = PIR1.0  
        Symbol TMR1IE = PIE1.0  
        Symbol GIEH = INTCON.7   
        Symbol GIEL = INTCON.6 
              
        Symbol TMR1ON = T1CON.0 

'****************************************************************            
'* Variables for NON critical subroutine updates                *
'****************************************************************  

        Dim TIMER_COUNT_REQ As Bit

        Dim TX_QUEUE_UPDATE_TIME As Byte        
        Dim TX_QUEUE_UPDATE_REQ_COUNTER As Byte
 
        Dim BASIC_DATA_UPDATE_TIME As Word        
        Dim BASIC_DATA_UPDATE_REQ_COUNTER As Word 
        
'Stagger update time(s) start so they do not all occur at once
        TX_QUEUE_UPDATE_REQ_COUNTER = 1
        BASIC_DATA_UPDATE_REQ_COUNTER = 5
        
'****************************************************************
'* Set non critical update times in multiples of 10mS as set by *
'* timed interrupt (timer 1)                                    *
'* If value is 100 then update time will be 100 x 10 = 1000mS   *
'****************************************************************   
        TX_QUEUE_UPDATE_TIME = 2     '20mS intervals for QUEUE TX
        BASIC_DATA_UPDATE_TIME = 100      '1 second interval for HEARTBEAT
'****************************************************************  

'****************************************************************
'CAN configuration settings                                     *
'**************************************************************** 
        
'Set configuration mode, no confirmation.            
        CANSetOperationMode [CAN_OP_MODE_CONFIG,NO]
        
'CAN configuration - accept valid standard messages.        
        CANInitialise [CAN_CONFIG_VALID_STD_MSG]      
        
'Set CAN node ID. (This must be unique to the network)
'                 (Can be set by dipswitches if preferred)
        CANSetNodeID [1]
        
'Set CAN message ID.        
        CANSetMsgID [0,STD]        

'****************************************************************
'Set CAN masks/filters                                          *
'**************************************************************** 

'Set CAN mask bits.         
        CANSetMask [CAN_MASK_B0,0,0,0,0,STD]
        CANSetMask [CAN_MASK_B1,0,0,0,0,STD]

'Set CAN filter bits.         
        CANSetFilter [CAN_FILTER_B0_F1,0,0,0,0,STD]
        CANSetFilter [CAN_FILTER_B0_F2,0,0,0,0,STD]        
        CANSetFilter [CAN_FILTER_B1_F1,0,0,0,0,STD]   
        CANSetFilter [CAN_FILTER_B1_F2,0,0,0,0,STD]        
        CANSetFilter [CAN_FILTER_B1_F3,0,0,0,0,STD]        
        CANSetFilter [CAN_FILTER_B1_F4,0,0,0,0,STD]             

'****************************************************************  
'* Set CAN BAUD rate.                                           *
'* The nodes can have different Xtal speeds but the BUS         * 
'* bitrate MUST be the same for all nodes.                      *   
'****************************************************************  

       '20Mhz / 125k BUS
       CANSetBaudRate [1,4,6,8,5,CAN_CONFIG_PHSEG2_PRG_ON & CAN_CONFIG_SAMPLE_THRICE & CAN_CONFIG_LINE_FILTER_OFF] 

'Use CAN node ID for pre determined delay before setting NORMAL mode
'This ensures that all nodes will not attempt to start transmitting at 
'the same time - Lowest ID will start first

'Only for STD message ID.        
        TX_NODE_NR.HighByte = TXB0SIDH
        TX_NODE_NR.LowByte = TXB0SIDL  
        TX_NODE_NR = TX_NODE_NR >> 5 'Shift right to get correct ID
        TX_NODE_NR = TX_NODE_NR & %0000000000011111 'Mask higher 11 bits
        DelayMS TX_NODE_NR
      
'Change back to normal operation mode, no confirmation.    
'For testing, set to listen and then use serial command to change to NORMAL, this should help until hardware proved and 
'can be powered up at the same time          
      ''  CANSetOperationMode [CAN_OP_MODE_NORMAL,NO]  
        CANSetOperationMode [CAN_OP_MODE_LISTEN,NO]    
        
'****************************************************************
'* End of CAN configuration settings                            *
'****************************************************************

'****************************************************************
'* Set up Timer 1                                               *
'****************************************************************     
        TIMER_1_BITS = %10000000   'Timer 1 configuration bits             
                                   'Timer 1 - 1:1 prescaler and act as timer                          
        TIMER_1_OFFSET = 15539     'Timer 1 offset for (10mS Interrupts) 
                                   'Configured for 20 Mhz xtal        

        IPR1 = %00000000           'Timer 1 low priority interrupt
        PIE1 = %00000001           'Timer 1 interrupt enable
        PIR1 = %00000000           '
        
'Include the interrupt routine
        Include "C:\JB_Test\CAN_INTERRUPT_INC.INC"    

'Include the queue routine
        Include "C:\JB_Test\CAN_QUE_INC.INC" 
        
'Include the CAN send routine         
        Include "C:\JB_Test\CAN_SEND_INC.INC"  
        
'Include the CAN send "FAST" routine           
        Include "C:\JB_Test\CAN_SEND_FAST_INC.INC"  
      
'****************************************************************
'* Start of Main Code                                           *
'****************************************************************
START:

'Set up TIMER1                            
        T1CON = TIMER_1_BITS
        TIMER1 = TIMER_1_OFFSET 'Load timer 1
        T1CON.0 = 1             'Start Timer running
        PIR1.0 = 0              'Clear timer 1 overflow bit
        PIE1 = 1                'Enable Timer1 interrupt        

        INTCON = %11000000      'Turn interrupts on 

'**************************************************************** 
'* Main program LOOP                                            *
'****************************************************************
LOOP:

        If  TIMER_COUNT_REQ = 1 Then GoSub TIMER_COUNT_UPDATE
        If  TX_QUEUE_UPDATE_REQ_COUNTER >= TX_QUEUE_UPDATE_TIME Then GoSub CAN_SEND
        If  BASIC_DATA_UPDATE_REQ_COUNTER >= BASIC_DATA_UPDATE_TIME Then GoSub BASIC_DATA  
        
        GoTo LOOP 
         
'****************************************************************
'* End of Main program LOOP                                     *                        
'****************************************************************

'Increment counters for non critical update subroutines. 
'These counters are reset to zero at the end of the routine after it has been called.
TIMER_COUNT_UPDATE:
        Inc H_BEAT
        Inc TX_QUEUE_UPDATE_REQ_COUNTER
        Inc BASIC_DATA_UPDATE_REQ_COUNTER
        TIMER_COUNT_REQ = 0
        Return

'****************************************************************
'* Basic Data -                                                 *                        
'****************************************************************

BASIC_DATA:
    
'Set parameters for CAN message ID.        
        MSGID_DATA = 1
        MSGID_TYPE = STD 
'Load into the queue          
        GoSub INSERT_IN_Q[1,H_BEAT,0x00,0x00,0x00,0x00,0x00,0x00,0x00]
      
        BASIC_DATA_UPDATE_REQ_COUNTER = 0
        
Return

'****************************************************************
'* CAN Error routine - Duplicate node detected  (FATAL)         *
'* If a duplicate node is detected then the program aborts ALL  *
'* pending messages, switches CAN module to LISTEN so as not to *
'* send any messages or acknowledge any and then terminates.    *
'* The fault is shown on the LCD (If available) and turns the   *
'* RED fault LED on.                                            *
'****************************************************************
CAN_DUPLICATE_ERROR: 
 
        CANAbortAll   'Abort ALL pending transmittions in ALL TX buffers

'Change operation mode to LISTEN, no confirmation              
        CANSetOperationMode [CAN_OP_MODE_LISTEN,NO]
        HRSOut "Duplicate Error", 10,10, 13 
        Cls                               'Clear screen 
        Print At 1,2, "CAN error"
        Print At 2,2, "Duplicate node ID "
        Print At 1,22, Dec RX_NODE_NR 
        Print At 2,22, "Detected "
               
        CAN_ERROR_LED = 1                 'Turn on RED LED for CAN error        
        Stop                              'FATAL error - PIC must be reset

